@@ -5,7 +5,8 @@ pub mod spg;
 pub mod message;
 pub mod connector;
 pub mod smartmodule;
-pub mod table;
+pub mod tableformat;
+pub mod derivedstream;
 
 pub use fluvio_stream_model::core;
 
@@ -30,7 +31,8 @@ pub mod extended {
         Partition,
         ManagedConnector,
         SmartModule,
-        Table,
+        TableFormat,
+        DerivedStream,
     }
 
     pub trait SpecExt: Spec {
